# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('text_field', models.TextField(blank=True)),
                ('text_field_two', models.TextField(blank=True)),
                ('char_field', models.CharField(max_length=255, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='TestModelTwo',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('text_field', models.TextField(blank=True)),
                ('char_field', models.CharField(max_length=255, blank=True)),
                ('url', models.URLField(blank=True)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from django.conf import settings
from django.core.management import call_command, CommandError
from django.test import TestCase, override_settings
from django.utils.six import StringIO

from models import TestModel


class TestWithoutAdminInstalled(TestCase):
    @classmethod
    def setUpTestData(cls):
        TestModel.objects.create(text_field="The quick brown fox")

    def test_admin_option_not_used_by_default(self):
        out = StringIO()
        call_command('grepdb', 'brown', 'tests.TestModel.text_field', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_admin_option_fails(self):
        with self.assertRaises(CommandError) as cm:
            call_command('grepdb', 'brown', 'tests.TestModel.text_field', '-s', '-l')
        self.assertEqual(cm.exception.message, u'Error: unrecognized arguments: -l')


@override_settings(
    INSTALLED_APPS=[
        'django.contrib.admin',
        'django.contrib.contenttypes',
        'django_grepdb',
        'django_grepdb.tests',
    ],
    ROOT_URLCONF='django_grepdb.tests.admin_urls'
)
class TestWithAdminInstalled(TestCase):
    @classmethod
    def setUpTestData(cls):
        TestModel.objects.create(text_field="The quick brown fox")

    def test_default_link_generation_output_with_defined_default(self):
        out = StringIO()
        call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object " \
                   "(pk=1)\x1b[0m\n\x1b[32mhttps://local.example.com/admin/tests/testmodel/1/\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    @override_settings(
        DJANGO_GREPDB_SITES = {
            'staging': 'https://staging.example.com',
            'production': 'https://example.com',
        }
    )
    def test_default_link_generation_output_without_defined_default(self):
        out = StringIO()
        call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object " \
                   "(pk=1)\x1b[0m\n\x1b[32mlocalhost:8000/admin/tests/testmodel/1/\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_option_without_argument_turns_off_link_generation(self):
        out = StringIO()
        call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', '-l', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_option_with_http_hostname(self):
        out = StringIO()
        call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', '-l', 'http://fox.example.com', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object " \
                   "(pk=1)\x1b[0m\n\x1b[32mhttp://fox.example.com/admin/tests/testmodel/1/\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_option_with_https_hostname(self):
        out = StringIO()
        call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', '-l', 'https://fox.example.com', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object " \
                   "(pk=1)\x1b[0m\n\x1b[32mhttps://fox.example.com/admin/tests/testmodel/1/\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_option_with_schemeless_hostname(self):
        with self.assertRaises(CommandError) as cm:
            call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', '-l', 'fox.example.com')
        msg = u'Reference fox.example.com is not recognised as a hostname and was not found in DJANGO_GREPDB_SITES'
        self.assertEqual(cm.exception.message, msg)

    def test_option_with_localhost(self):
        out = StringIO()
        call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', '-l', 'localhost:4000', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object " \
                   "(pk=1)\x1b[0m\n\x1b[32mlocalhost:4000/admin/tests/testmodel/1/\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_option_with_sites_key(self):
        out = StringIO()
        call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', '-l', 'production', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object " \
                   "(pk=1)\x1b[0m\n\x1b[32mhttps://example.com/admin/tests/testmodel/1/\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_option_with_mixed_arguments(self):
        out = StringIO()
        call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', '-l', 'staging', 'production', 'default',
                     'https://dev.example.com', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n" \
                   "\x1b[32mhttps://staging.example.com/admin/tests/testmodel/1/\x1b[0m\n" \
                   "\x1b[32mhttps://example.com/admin/tests/testmodel/1/\x1b[0m\n" \
                   "\x1b[32mhttps://local.example.com/admin/tests/testmodel/1/\x1b[0m\n" \
                   "\x1b[32mhttps://dev.example.com/admin/tests/testmodel/1/\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    @override_settings()
    def test_option_without_sites_setting(self):
        del settings.DJANGO_GREPDB_SITES
        with self.assertRaises(CommandError) as cm:
            call_command('grepdb', 'quick', 'tests.TestModel.text_field', '-s', '-l', 'fox.example.com')
        msg = u'Reference fox.example.com is not recognised as a hostname and DJANGO_GREPDB_SITES is not ' \
              'configured in settings'
        self.assertEqual(cm.exception.message, msg)



# -*- coding: utf-8 -*-
from django.core.management import call_command, CommandError
from django.test import TestCase
from django.utils.six import StringIO

from models import TestModel


class TestShowValues(TestCase):
    @classmethod
    def setUpTestData(cls):
        TestModel.objects.create(text_field="Lorem ipsum dolor sit amet, eu eam option expetendis. An equidem " \
                                            "noluisse appareat his. Feugiat meliore vix ex, vel ad elit feugiat.")
        TestModel.objects.create(text_field="In qui doming evertitur, cum dicam vituperatoribus id. Insolens " \
                                            "assueverit quo an, et case choro minimum usu, te sit mediocrem.\n" \
                                            "Te placerat deserunt cum, et per enim mutat audire, putent graeco " \
                                            "eligendi ne vix. Ut nisl ornatus nec, discere fierent eu has.")
        TestModel.objects.create(text_field="Cu usu molestie invidunt molestie usu cu.")

    def test_default_format_on_single_line_content(self):
        out = StringIO()
        call_command('grepdb', 'Feugiat', 'tests.TestModel', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n" \
                   "Lorem ipsum dolor sit amet, eu eam option expetendis. An equidem noluisse appareat his. " \
                   "\x1b[43m\x1b[30mFeugiat\x1b[0m meliore vix ex, vel ad elit feugiat.\n\n"
        self.assertEqual(out.getvalue(), expected)

    def test_default_format_on_multi_line_content(self):
        out = StringIO()
        call_command('grepdb', 'dicam', 'tests.TestModel', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=2)\x1b[0m\n" \
                   "In qui doming evertitur, cum \x1b[43m\x1b[30mdicam\x1b[0m vituperatoribus id. Insolens " \
                   "assueverit quo an, et case choro minimum usu, te sit mediocrem.\n\n"
        self.assertEqual(out.getvalue(), expected)

    def test_show_all_on_multi_line_content(self):
        out = StringIO()
        call_command('grepdb', 'dicam', 'tests.TestModel', '-sa', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=2)\x1b[0m\n" \
                   "In qui doming evertitur, cum \x1b[43m\x1b[30mdicam\x1b[0m vituperatoribus id. Insolens " \
                   "assueverit quo an, et case choro minimum usu, te sit mediocrem.\n" \
                   "Te placerat deserunt cum, et per enim mutat audire, putent graeco " \
                   "eligendi ne vix. Ut nisl ornatus nec, discere fierent eu has.\n\n"
        self.assertEqual(out.getvalue(), expected)

    def test_show_surrounding_chars_on_single_line_content(self):
        out = StringIO()
        call_command('grepdb', 'Feugiat', 'tests.TestModel', '-s5', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n" \
                   "his. \x1b[43m\x1b[30mFeugiat\x1b[0m meli\n\n"
        self.assertEqual(out.getvalue(), expected)

    def test_show_surrounding_chars_across_lines(self):
        out = StringIO()
        call_command('grepdb', 'mediocrem\.', 'tests.TestModel', '-s5', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=2)\x1b[0m\n" \
                   "sit \x1b[43m\x1b[30mmediocrem.\x1b[0m\nTe p\n\n"
        self.assertEqual(out.getvalue(), expected)

    def test_invalid_show_option(self):
        with self.assertRaises(CommandError) as cm:
            call_command('grepdb', 'Feugiat', '-sm')
        msg = "Error: argument --show-values/-s: Show values style must be one of 'a, l' or an integer"
        self.assertEqual(cm.exception.message, msg)

    def test_show_surrounding_chars_with_multiple_joined_matches(self):
        """Output should highlight every match accurately"""
        out = StringIO()
        call_command('grepdb', 'molestie', 'tests.TestModel', '-s10', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=3)\x1b[0m\n"\
                   "\x1b[43m\x1b[30mmolestie\x1b[0m invidunt \x1b[43m\x1b[30mmolestie\x1b[0m usu cu.\n\n"
        self.assertEqual(out.getvalue(), expected)

    def test_show_surrounding_chars_with_overlapping_matches(self):
        """Output should highlight every match accurately"""
        out = StringIO()
        call_command('grepdb', '.{5}molestie.{5}', 'tests.TestModel', '-s5', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=3)\x1b[0m\n"\
                   "\x1b[43m\x1b[30m usu molestie invi\x1b[0m\x1b[43m\x1b[30mdunt molestie usu \x1b[0mcu.\n\n"
        self.assertEqual(out.getvalue(), expected)

    def test_default_format_on_single_line_content_case_insensitive(self):
        out = StringIO()
        call_command('grepdb', 'MELIORE', 'tests.TestModel', '-i', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n" \
                   "Lorem ipsum dolor sit amet, eu eam option expetendis. An equidem noluisse appareat his. " \
                   "Feugiat \x1b[43m\x1b[30mmeliore\x1b[0m vix ex, vel ad elit feugiat.\n\n"
        self.assertEqual(out.getvalue(), expected)

    def test_show_all_on_multi_line_content_case_insensitive(self):
        out = StringIO()
        call_command('grepdb', 'DICAM', 'tests.TestModel', '-sa', '-i', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=2)\x1b[0m\n" \
                   "In qui doming evertitur, cum \x1b[43m\x1b[30mdicam\x1b[0m vituperatoribus id. Insolens " \
                   "assueverit quo an, et case choro minimum usu, te sit mediocrem.\n" \
                   "Te placerat deserunt cum, et per enim mutat audire, putent graeco " \
                   "eligendi ne vix. Ut nisl ornatus nec, discere fierent eu has.\n\n"
        self.assertEqual(out.getvalue(), expected)

    def test_show_surrounding_chars_on_single_line_content_case_insensitive(self):
        out = StringIO()
        call_command('grepdb', 'MELIORE', 'tests.TestModel', '-s5', '-i', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n" \
                   "giat \x1b[43m\x1b[30mmeliore\x1b[0m vix\n\n"
        self.assertEqual(out.getvalue(), expected)



from django.conf.urls import include, url
from django.contrib import admin

from models import TestModel, TestModelTwo

admin.site.register(TestModel)
admin.site.register(TestModelTwo)


urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),
]



# -*- coding: utf-8 -*-
from django.core.management import call_command
from django.test import TestCase
from django.utils.six import StringIO

from models import TestModel


class TestBasicsEndToEnd(TestCase):
    @classmethod
    def setUpTestData(cls):
        TestModel.objects.create(text_field="The quick brown fox",
                                 text_field_two="jumped over the lazy brown dog")
        TestModel.objects.create(text_field="The fox and the cat were not lazy")
        TestModel.objects.create(text_field="The CAT was not lazy")

    def test_minimal_output(self):
        out = StringIO()
        call_command('grepdb', 'brown', 'tests.TestModel.text_field', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_minimal_output_two_matches(self):
        out = StringIO()
        call_command('grepdb', 'fox', 'tests.TestModel.text_field', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n\x1b" \
                   "[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n\x1b[1m\x1b[32mTestModel object (pk=2)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_minimal_output_two_fields(self):
        out = StringIO()
        call_command('grepdb', 'brown', 'tests.TestModel.text_field', 'tests.TestModel.text_field_two',
                     '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n\x1b[1m\x1b" \
                   "[32mTestModel object (pk=1)\x1b[0m\n\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models." \
                   "TestModel'> text_field_two\x1b[0m\n\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_minimal_output_case_sensitive_by_default(self):
        out = StringIO()
        call_command('grepdb', 'cat', 'tests.TestModel.text_field', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> " \
                   "text_field\x1b[0m\n\x1b[1m\x1b[32mTestModel object (pk=2)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_minimal_output_case_insensitive(self):
        out = StringIO()
        call_command('grepdb', 'cat', 'tests.TestModel.text_field', '-s', '-i', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n\x1b[1m\x1b" \
                   "[32mTestModel object (pk=2)\x1b[0m\n\x1b[1m\x1b[32mTestModel object (pk=3)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_minimal_output_regex_search(self):
        out = StringIO()
        call_command('grepdb', 'The.* fox', 'tests.TestModel.text_field', '-s', '-i', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n\x1b[1m\x1b" \
                   "[32mTestModel object (pk=1)\x1b[0m\n\x1b[1m\x1b[32mTestModel object (pk=2)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_minimal_output_regex_search_two(self):
        out = StringIO()
        call_command('grepdb', 'The.+ fox', 'tests.TestModel.text_field', '-s', '-i', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n\x1b[1m\x1b" \
                   "[32mTestModel object (pk=1)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)



# -*- coding: utf-8 -*-
from django.core.management import call_command, CommandError
from django.test import TestCase
from django.utils.six import StringIO

from models import TestModelTwo


class TestPresets(TestCase):
    @classmethod
    def setUpTestData(cls):
        TestModelTwo.objects.create(text_field="The quick brown fox")
        TestModelTwo.objects.create(char_field="The quick brown fox")
        TestModelTwo.objects.create(url="http://example.com/fox/")

    def test_field_type_defaults_to_text_fields(self):
        out = StringIO()
        call_command('grepdb', 'fox', 'tests.TestModelTwo', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=1)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_special_charfield_detection_option(self):
        """-c option should detect the CharField and the URLField (subclass of CharField)"""
        out = StringIO()
        call_command('grepdb', 'fox', 'tests.TestModelTwo', '-c', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> char_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=2)\x1b[0m\n" \
                   "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> url\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=3)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_named_field_type_detection_option(self):
        out = StringIO()
        call_command('grepdb', 'fox', 'tests.TestModelTwo', '-f', 'CharField', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> char_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=2)\x1b[0m\n" \
                   "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> url\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=3)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_multiple_field_type_detection_options(self):
        out = StringIO()
        call_command('grepdb', 'fox', 'tests.TestModelTwo', '-c', '-t', '-s', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=1)\x1b[0m\n" \
                   "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> char_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=2)\x1b[0m\n" \
                   "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> url\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=3)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)



# -*- coding: utf-8 -*-
from django.core.management import CommandError
from django.test import TestCase, override_settings
from django.utils.six import StringIO

from ..management import call_command
from models import TestModel, TestModelTwo


class TestWithoutPresets(TestCase):
    @classmethod
    def setUpTestData(cls):
        TestModel.objects.create(text_field="The quick brown fox", text_field_two="jumped over the lazy dog")

    def test_presets_option_not_used_by_default(self):
        out = StringIO()
        call_command('grepdb', 'brown', stdout=out)
        expected = ""
        self.assertEqual(out.getvalue(), expected)

    def test_preset_option_fails(self):
        with self.assertRaises(CommandError) as cm:
            call_command('grepdb', 'brown', '-p', 'model_one')
        msg = u'Preset specified but DJANGO_GREPDB_PRESETS is not configured in settings'
        self.assertEqual(cm.exception.message, msg)


@override_settings(
    DJANGO_GREPDB_PRESETS=[
        {'model_one': {'identifiers': ['tests.TestModel']}}
    ]
)
class TestWithMisconfiguredSetting(TestCase):
    def test_preset_option_fails(self):
        with self.assertRaises(CommandError) as cm:
            call_command('grepdb', 'brown', '-p', 'model_one')
        msg = u'DJANGO_GREPDB_PRESETS is not a dict-like object'
        self.assertEqual(cm.exception.message, msg)


@override_settings(
    DJANGO_GREPDB_PRESETS={
        'model_one': {'identifiers': ['tests.TestModel']},
        'model_one_case_insensitive': {'identifiers': ['tests.TestModel'], 'ignore_case': True},
        'model_two_multiple_fields': {'identifiers': ['tests.TestModelTwo'], 'field_type': ['CharField', 'TextField']},
        'both_models': {'identifiers': ['tests.TestModelTwo', 'tests.TestModel']},
        'preset_is_a_list': ['tests.TestModel', 'ignore_case']
    }
)
class TestPresets(TestCase):
    @classmethod
    def setUpTestData(cls):
        TestModel.objects.create(text_field="The quick brown fox", text_field_two="jumped over the lazy dog")
        TestModel.objects.create(text_field="THE QUICK BROWN FOX")
        TestModelTwo.objects.create(text_field="The dog was lazy")
        TestModelTwo.objects.create(char_field="The fox was quick and brown")

    def test_error_raised_when_using_default_call_command(self):
        from django.core.management import call_command
        with self.assertRaises(CommandError) as cm:
            call_command('grepdb', 'brown', '-p', 'model_one')
        msg = "--preset mode is not compatible with django.core.management.call_command: " \
              "you need to use django_grepdb.management.call_command instead"
        self.assertEqual(cm.exception.message, msg)

    def test_identifier_from_preset(self):
        out = StringIO()
        call_command('grepdb', 'brown', '-s', '-p', 'model_one', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_identifier_and_toggle_from_preset(self):
        out = StringIO()
        call_command('grepdb', 'brown', '-s', '-p', 'model_one_case_insensitive', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=2)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_identifier_and_field_types_from_preset(self):
        out = StringIO()
        call_command('grepdb', 'was', '-s', '-p', 'model_two_multiple_fields', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=1)\x1b[0m\n" \
                   "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> char_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=2)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_multiple_identifiers_from_preset(self):
        out = StringIO()
        call_command('grepdb', 'dog', '-s', '-p', 'both_models', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModelTwo'> text_field\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModelTwo object (pk=1)\x1b[0m\n" \
                   "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field_two\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_command_line_overrides_preset(self):
        out = StringIO()
        call_command('grepdb', 'dog', 'tests.TestModel', '-s', '-p', 'both_models', stdout=out)
        expected = "\x1b[1m\x1b[36m\n<class 'django_grepdb.tests.models.TestModel'> text_field_two\x1b[0m\n" \
                   "\x1b[1m\x1b[32mTestModel object (pk=1)\x1b[0m\n"
        self.assertEqual(out.getvalue(), expected)

    def test_misconfigured_preset(self):
        with self.assertRaises(CommandError) as cm:
            call_command('grepdb', 'brown', '-p', 'preset_is_a_list')
        msg = 'Preset "preset_is_a_list" is not a dict-like object'
        self.assertEqual(cm.exception.message, msg)

    def test_unknown_preset(self):
        with self.assertRaises(CommandError) as cm:
            call_command('grepdb', 'brown', '-p', 'what_preset')
        msg = 'Preset "what_preset" not found in DJANGO_GREPDB_PRESETS. Available values are: ' \
              'model_two_multiple_fields, preset_is_a_list, model_one, model_one_case_insensitive, both_models'
        self.assertEqual(cm.exception.message, msg)



import argparse
import re

import colorama
from django.apps import apps
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand, CommandError
from django.core.urlresolvers import reverse
from termcolor import colored


def show_values_style(arg):
    special_choices = ['a', 'l']
    if arg in special_choices:
        return arg
    try:
        return int(arg)
    except ValueError:
        raise argparse.ArgumentTypeError("Show values style must be one of '{values}' or an integer".format(
            values=', '.join(special_choices)))


class Command(BaseCommand):
    help = 'Provides a grep-like command line interface for searching objects in the database'

    def add_arguments(self, parser):
        parser.add_argument('pattern', type=str, help='Pattern to search for')
        parser.add_argument('identifiers', nargs='*', type=str, help='Identifier of a model or field')
        parser.add_argument('--show-values', '-s', nargs='?', type=show_values_style, default='l',
                            help='Turn off showing matching values (default is any line containing a match), ' \
                            'or provide the mode "a" to show the entire field ' \
                            'or an integer to show that many characters either side of a match.')
        parser.add_argument('--ignore-case', '-i', action='store_true', help='Match case-insensitively')
        parser.add_argument('--find-text-fields', '-t', dest='field_type', action='append_const', const='TextField',
                            help='Search all TextField fields (and subclasses) on a model if no field is specified')
        parser.add_argument('--find-char-fields', '-c', dest='field_type', action='append_const', const='CharField',
                            help='Search all CharField fields (and subclasses) on a model if no field is specified')
        parser.add_argument('--find-fields', '-f', dest='field_type', action='append', type=str,
                            help='Search all fields of this type (and subclasses) on a model if no field is specified')
        parser.add_argument('--preset', '-p', help='The name of a preset configuration in DJANGO_GREPDB_PRESETS. ' \
                            'DJANGO_GREPDB_PRESETS should be a dict of dicts, with each config dict providing ' \
                            'default values for any number of parser args.')
        if apps.is_installed('django.contrib.admin'):
            parser.add_argument('--admin-links', '-l', dest='admin_hostname', nargs='*', default=['default'],
                                help='Generate admin links. Defaults to true, using http://localhost:8000/ as hostname. ' \
                                'Can be passed one or more hostnames to use instead. If DJANGO_GREPDB_SITES is a ' \
                                'dict defined in settings, the value of the "default" key will be used as default, ' \
                                'and keys from it can also be passed to use their values as hostnames. ' \
                                'Links can be disabled by using this argument without any values.')
        self.parser = parser

    def handle(self, **options):
        colorama.init()
        preset = self.get_preset(options['preset'])
        if preset:
            self.parser.set_defaults(**preset)
            # re-parse the command line arguments with new defaults in place
            try:
                options = vars(self.parser.parse_args(self.raw_args))
            except AttributeError:
                if not self._called_from_command_line:
                    # regular call_command doesn't store raw_args
                    msg = '--preset mode is not compatible with django.core.management.call_command: you need to ' \
                          'use django_grepdb.management.call_command instead'
                    raise CommandError(msg)
                else:
                    # if it was called from the command line, the problem is something unknown
                    raise
        self.pattern = options['pattern']
        self.ignore_case = options['ignore_case']
        self.show_values = options.get('show_values', False)
        self.field_type = options['field_type'] or ['TextField']
        self.admin_hostnames = self.get_admin_hostnames(options)

        identifiers = options['identifiers']
        queries = self.get_queries(identifiers)
        for query in queries:
            results = self.search(query)
            if results.exists():
                self.stdout.write(colored(u'\n{model} {field}'.format(model=query['manager'].model, field=query['field_name']),
                                          'cyan', attrs=['bold']))
                for result in results:
                    self.stdout.write(colored(u'{result} (pk={result.pk})'.format(result=result), 'green', attrs=['bold']))
                    if self.admin_hostnames:
                        self.stdout.write(self.get_admin_links(result))
                    if self.show_values is not None:  # can't be a truthiness check, as zero is different from no show
                        self.stdout.write(self.get_value(result, query))

    def run_from_argv(self, argv):
        # store raw args so that we can re-parse them with new defaults if preset mode is used
        self.raw_args = argv[2:]
        super(Command, self).run_from_argv(argv)

    def get_admin_hostnames(self, options):
        from_options = options.get('admin_hostname', False)
        if not from_options:
            return
        from django.contrib.admin import site as admin_site
        self.admin_site = admin_site
        hostnames = []
        for reference in from_options:
            hostnames.append(self.get_admin_hostname(reference))
        return hostnames

    def get_admin_hostname(self, reference):
        """Treats the reference as a hostname if it contains either 'http' or 'localhost'.
        If it contains neither, looks up the reference in settings.DJANGO_GREPDB_SITES
        """
        if 'http' in reference or 'localhost' in reference:
            return reference
        try:
            hostname = self.get_admin_hostname_from_settings(reference)
        except CommandError:
            if reference == 'default':
                hostname = 'localhost:8000'
            else:
                raise
        return hostname

    def get_admin_hostname_from_settings(self, reference):
        try:
            sites = getattr(settings, 'DJANGO_GREPDB_SITES')
        except AttributeError:
            msg = u'Reference {} is not recognised as a hostname and DJANGO_GREPDB_SITES is not configured in settings'
            raise CommandError(msg.format(reference))
        try:
            hostname = sites[reference]
        except KeyError:
            msg = u'Reference {} is not recognised as a hostname and was not found in DJANGO_GREPDB_SITES'
            raise CommandError(msg.format(reference))
        return hostname

    def get_preset(self, preset_name):
        if not preset_name:
            return None
        try:
            presets = getattr(settings, 'DJANGO_GREPDB_PRESETS')
        except AttributeError:
            raise CommandError(u'Preset specified but DJANGO_GREPDB_PRESETS is not configured in settings')
        try:
            preset = presets[preset_name]
        except TypeError:
            msg = u'DJANGO_GREPDB_PRESETS is not a dict-like object'
            raise CommandError(msg)
        except KeyError:
            msg = u'Preset "{preset_name}" not found in DJANGO_GREPDB_PRESETS. Available values are: {values}'
            raise CommandError(msg.format(preset_name=preset_name, values=', '.join(presets.keys())))
        try:
            preset.keys()
        except AttributeError:
            msg = u'Preset "{preset_name}" is not a dict-like object'
            raise CommandError(msg.format(preset_name=preset_name))
        return preset

    def get_queries(self, identifiers):
        queries = []
        for identifier in identifiers:
            queries.extend(self.get_queries_for_identifier(identifier))
        return queries

    def get_queries_for_identifier(self, identifier):
        model, field_names = self.parse_identifier(identifier)
        queries = []
        for field_name in field_names:
            params = self.get_queryset_params(field_name)
            queries.append(dict(manager=model._default_manager, params=params, field_name=field_name))
        return queries

    def search(self, query):
        return query['manager'].filter(**query['params'])

    def parse_identifier(self, identifier):
        parts = identifier.split('.')
        app_label, model_name = parts[:2]
        field_names = parts[2:]
        model = apps.get_model(app_label, model_name)
        if not field_names:
            field_names = self.get_field_names_for_model(model)
        return (model, field_names)

    def get_field_names_for_model(self, model):
        return [field.name for field in model._meta.fields if field.get_internal_type() in self.field_type]

    def get_queryset_params(self, field_name):
        lookup_type = 'regex'
        if self.ignore_case:
            lookup_type = 'i' + lookup_type
        return {'{field_name}__{lookup_type}'.format(field_name=field_name, lookup_type=lookup_type): self.pattern}

    def get_value(self, result, query):
        text = getattr(result, query['field_name'])
        show_values = self.show_values
        if show_values == 'a':
            return self.get_value_all(text)
        elif show_values == 'l':
            return self.get_value_line(text)
        else:
            return self.get_value_surrounded(text)

    def get_value_all(self, text):
        regex_args = [self.pattern, text, re.DOTALL]
        if self.ignore_case:
            regex_args[2] += re.IGNORECASE
        matches = [m.span() for m in re.finditer(*regex_args)]
        value = u''
        end_of_previous = 0
        for start, end in matches:
            value = value + text[end_of_previous:start] + colored(text[start:end], 'grey', 'on_yellow')
            end_of_previous = end
        value = value + text[end_of_previous:] + '\n\n'
        return value

    def get_value_line(self, text):
        value = u''
        for line in text.splitlines():
            regex_args = [self.pattern, line]
            if self.ignore_case:
                regex_args.append(re.IGNORECASE)
            matches = [m.span() for m in re.finditer(*regex_args)]
            if matches:
                end_of_previous = 0
                for start, end in matches:
                    value = value + line[end_of_previous:start] + colored(line[start:end], 'grey', 'on_yellow')
                    end_of_previous = end
                value = value + line[end_of_previous:] + '\n\n'
        return value

    def get_value_surrounded(self, text):
        regex_args = [self.pattern, text]
        if self.ignore_case:
            regex_args.append(re.IGNORECASE)
        matches = re.findall(*regex_args)
        chars = self.show_values
        matches = [m.span() for m in re.finditer(*regex_args)]
        value = u''
        end_of_previous = 0
        for start, end in matches:
            if end_of_previous and end_of_previous > start:
                value = value[:start - end_of_previous]
            elif end_of_previous and end_of_previous > start - chars:
                value += text[end_of_previous:start]
            else:
                value += '\n' + text[start - chars:start]
            value += colored(text[start:end], 'grey', 'on_yellow') + text[end:end + chars]
            end_of_previous = end + chars
        value = value.strip() + '\n\n'
        return value

    def get_admin_links(self, result):
        content_type = ContentType.objects.get_for_model(result)
        admin_url_pattern = 'admin:{app}_{model}_change'.format(app=content_type.app_label, model=content_type.model)
        relative_url = reverse(admin_url_pattern, args=[result.pk])
        return '\n'.join([colored(hostname + relative_url, 'green') for hostname in self.admin_hostnames])

    def get_version(self):
        from ...version import VERSION
        return VERSION



from django.core.management import CommandError, BaseCommand, get_commands, load_command_class


def call_command(name, *args, **options):
    """
    Calls the given command, with the given options and args/kwargs.

    This is the primary API you should use for calling specific commands.

    Some examples:
        call_command('migrate')
        call_command('shell', plain=True)
        call_command('sqlmigrate', 'myapp')

    Copy of the function from django.core.management. In addition to the
    standard functionality, stores the raw args on the command instance.
    """
    # Load the command object.
    try:
        app_name = get_commands()[name]
    except KeyError:
        raise CommandError("Unknown command: %r" % name)

    if isinstance(app_name, BaseCommand):
        # If the command is already loaded, use it directly.
        command = app_name
    else:
        command = load_command_class(app_name, name)

    # Store the raw args on the command so they can be used later if needed
    command.raw_args = args

    # Simulate argument parsing to get the option defaults (see #10080 for details).
    parser = command.create_parser('', name)
    if command.use_argparse:
        # Use the `dest` option name from the parser option
        opt_mapping = {sorted(s_opt.option_strings)[0].lstrip('-').replace('-', '_'): s_opt.dest
                       for s_opt in parser._actions if s_opt.option_strings}
        arg_options = {opt_mapping.get(key, key): value for key, value in options.items()}
        defaults = parser.parse_args(args=args)
        defaults = dict(defaults._get_kwargs(), **arg_options)
        # Move positional args out of options to mimic legacy optparse
        args = defaults.pop('args', ())
    else:
        # Legacy optparse method
        defaults, _ = parser.parse_args(args=[])
        defaults = dict(defaults.__dict__, **options)
    if 'skip_checks' not in options:
        defaults['skip_checks'] = True

    return command.execute(*args, **defaults)



from django.db import models


class TestModel(models.Model):
    text_field = models.TextField(blank=True)
    text_field_two = models.TextField(blank=True)
    char_field = models.CharField(blank=True, max_length=255)

    class Meta:
        app_label = 'tests'


class TestModelTwo(models.Model):
    text_field = models.TextField(blank=True)
    char_field = models.CharField(blank=True, max_length=255)
    url = models.URLField(blank=True)

    class Meta:
        app_label = 'tests'



import os
from setuptools import find_packages, setup

from django_grepdb.version import VERSION

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-grep-db',
    version=VERSION,
    packages=find_packages(),
    install_requires=['django>=1.8', 'colorama>=0.3.3', 'termcolor>=1.1.0'],

    description='A simple Django app for command-line searching via the ORM',
    long_description=README,
    keywords='django search grep cli',
    license='MIT',
    author='Michael Blatherwick',
    author_email='michael.blatherwick@exeter.oxon.org',
    url='https://github.com/exonian/django-grep-db',
)
